{{
        VGA 640 x 480 1-bit with 2-bit inset driver.
           (c) Copyright 2012 Philip C. Pilgrim
         "vga_driver" portion loosely derived from
           VGA64_PIXEngine by Kwabena W. Agyeman
             See end of file for terms of use.
}}

CON

  _clkmode      = xtal1 + pll16x
  _xinfreq      = 5_000_000

'Size of the color palette FIFO buffer:

  BUF_SIZE      = 511

'Pin groups:
  
  PROP_BOE_0    = 0 'PO - P7 
  PROP_BOE_1    = 1 'P8 - P15
  DEMO_BOARD    = 2 'P16 - P23

'SD Card pins:

  DO_PIN        = 22
  CLK_PIN       = 23
  DI_PIN        = 24
  nCS_PIN       = 25

  XBEE_PIN      = 0

OBJ

  sd    : "fsrw_mini_rd2-2"
  sio   : "basic_sio"

VAR

  byte  slide_no, pstate

PUB main

  start_vga(PROP_BOE_1)
  sio.start_explicit(XBEE_PIN, 30, 9600)
  waitcnt(cnt + clkfreq)
  sd.mount_explicit(DO_PIN, CLK_PIN, DI_PIN, nCS_PIN)

  repeat
    show_slide (get_remote)

PUB start_vga(pin_group) | pix_clk

  wipe_data := 32
  pin_group := ((pin_group <# 3) #> 0)
  dira0 := ($FF << (pin_group << 3))
  vcfg0 := %0_01_1 << 28 | pin_group << 9 | $ff
  cvsync_mask := 1 << (pin_group << 3)
   
  pix_clk := constant(25_175_000 / 4)
  frqa0 := 1
   
  repeat 32
    pix_clk <<= 1
    frqa0 <-= 1
    if(pix_clk => clkfreq)
      pix_clk -= clkfreq
      frqa0++
   
  deq_ptr := buffer_addr := @color_buf
  cognew(@ovl_cdriver, @buffer_addr)
  cognew(@vga_driver, @buffer_addr)

PUB get_remote | state

repeat
  repeat until (sio.in == $7e)
  repeat 11
    sio.in
  state := sio.in
  if (state <> pstate)
    pstate := state
    if (state <> 3)
      return state == 2

PUB show_slide(fwd)

  if (fwd)
    slide_no++
  elseif (slide_no > 1)
    slide_no--

  repeat
    slide_num[0] := slide_no / 100 + "0"
    slide_num[1] := slide_no / 10 // 10 + "0"
    slide_num[2] := slide_no // 10 + "0"
    ifnot (\sd.popen(@file_name, "r"))
      quit
    slide_no--
  repeat while ++wipe_data <# 32 < 32
    waitcnt(cnt + (clkfreq >> 7))
  sd.pread(@img_data, sd.get_filesize)
  sd.pclose
  repeat wipe_data from 31 to 0
    waitcnt(cnt + (clkfreq >> 7))

DAT

file_name     byte      "slide"
slide_num     byte      "000.pvd", 0

wipe_data     long      0
deq_ptr       long      0
color_buf     


'-------[ 640 x 480 x 1-bit VGA Driver ]---------------------------------------

              org       0

vga_driver    rdlong    buf_addr,par
              mov       deq_addr,buf_addr
              sub       deq_addr,#4
              mov       wipe_addr,buf_addr
              sub       wipe_addr,#8
              mov       buf_index,buf_addr
              mov       buf_end,#BUF_SIZE
              shl       buf_end,#2
              add       buf_end,buf_addr
              
              mov       vcfg,vcfg0              ' Setup video hardware.
              mov       frqa,frqa0         
              movi      ctra,#%0_00001_101
              mov       dira,dira0

'       Vertical Front Porch

frame_loop    mov       counter,#10             ' Set loop counter.

front_porch   mov       vscl,blank_vscl         ' Invisible lines.
              waitvid   hsync_colors,#0                         '

              mov       vscl,invis_vscl         ' Horizontal Sync.
              waitvid   hsync_colors,sync_pix    
              djnz      counter,#front_porch    ' Repeat # times.

              mov       buf_index,buf_addr
              wrlong    buf_index,deq_addr

'       Vertical Sync

              mov       counter,#4              ' Set loop counter.

vsync_loop    mov       vscl,blank_vscl         ' Invisible lines.
              waitvid   vsync_colors,#0          

              mov       vscl,invis_vscl         ' Vertical sync.
              waitvid   vsync_colors,sync_pix    
              djnz      counter,#vsync_loop     ' Repeat.

'       Vertical Back Porch

              mov       counter,#31             ' Set loop counter.

back_porch    mov       vscl,blank_vscl         ' Invisible lines.
              waitvid   hsync_colors,#0          

              mov       vscl,invis_vscl         ' Horizontal Sync.
              waitvid   hsync_colors,sync_pix    
              djnz      counter,#back_porch     ' Repeat # times.
              
'       Active Video

              mov       pix2_addr,par
              add       pix2_addr,#4
              rdlong    pix4_addr,pix2_addr
              add       pix2_addr,#4
              add       pix4_addr,pix2_addr      

              mov       line_ctr,#480
              mov       pix2_ptr,pix2_addr
              rdlong    y_alt,pix2_ptr
              add       pix2_ptr,#4
              mov       y_ctr,#1
              rdlong    alt_colors,buf_index
              or        alt_colors,all_black
              add       buf_index,#4
              mov       pix4_ptr,pix4_addr
              rdlong    wipe,wipe_addr

              maxs      wipe,#32
              mins      wipe,#0
              tjz       wipe,#line_loop
        
              mov       counter,wipe
              shl       wipe,#4
              sub       wipe,counter
              sub       line_ctr,wipe

wipe_lp       mov       counter,#20
              mov       vscl,vis4_vscl

wipe_line     waitvid   wipe_color,#0
              djnz      counter,#wipe_line

              mov       vscl,invis_vscl         ' Set/Reset the video scale.
              waitvid   hsync_colors,sync_pix   ' Horizontal Sync.
              djnz      wipe,#wipe_lp

              tjz       line_ctr,#frame_loop             

line_loop     djnz      y_ctr,#intra_block

              mov       y_ctr,#15
              mov       alt_master,#0
              shl       y_alt,#1 wc
        if_nc jmp       #intra_block
        
              rdlong    alt_master,pix2_ptr
              add       pix2_ptr,#4
       
intra_block   rdlong    reg_mask,pix2_ptr
              add       pix2_ptr,#4
              mov       reg_colors,reg_mask
              shr       reg_colors,#6
              and       reg_colors,#%111111
              shl       reg_colors,#10
              mov       pixels,reg_mask
              and       pixels,#%111111
              shl       pixels,#2
              or        reg_colors,pixels
              mov       pixels,reg_colors
              shl       pixels,#16
              or        reg_colors,pixels
              or        reg_colors,all_black

              mov       counter,#20             ' 20 x 32-bit tiles = 640 pixels
              mov       alt_mask,alt_master
              shl       alt_mask,#1 wc
              muxc      $,$ wz,nr
              shl       reg_mask,#1 wc   

'       Visible Video

pix_lp  if_z  jmp       #reg_pix

alt_pix       mov       vscl,vis4_vscl
              rdlong    pixels,pix4_ptr
              add       pix4_ptr,#4
              waitvid   alt_colors,pixels
              rdlong    alt_colors,buf_index
              or        alt_colors,all_black
              add       buf_index,#4
              cmp       buf_index,buf_end wz
        if_z  mov       buf_index,buf_addr
              wrlong    buf_index,deq_addr 
              shl       alt_mask,#1 wc
              muxc      $,$ wz,nr
              shl       reg_mask,#1 wc
              djnz      counter,#pix_lp

              jmp       #do_sync

reg_pix       mov       vscl,vis2_vscl
        if_c  rdlong    pixels,pix2_ptr
        if_nc mov       pixels,#0
              waitvid   reg_colors,pixels
        if_c  add       pix2_ptr,#4
              shr       pixels,#1
              shl       alt_mask,#1 wc
              muxc      $,$ wz,nr
              shl       reg_mask,#1 wc
              waitvid   reg_colors,pixels
              djnz      counter,#pix_lp         ' Repeat.

'       Horizontal Front Porch, Sync, and Back Porch

do_sync       mov       vscl,invis_vscl         ' Set/Reset the video scale.
              waitvid   hsync_colors,sync_pix   ' Horizontal Sync.
              djnz      line_ctr,#line_loop 


              jmp       #frame_loop             ' Loop.

'       Data

invis_vscl    long      (16 << 12) + 160        ' Scaling for inactive video.
vis2_vscl     long      (1 << 12) + 16          ' Scaling for 2-color active video.
vis4_vscl     long      (2 << 12) + 32          ' Scaling for 4-color active video.
blank_vscl    long      640                     ' Blank scanline pixel length.
sync_pix      long      $00_00_3f_fc            ' F-porch, h-sync, and b-porch.
hsync_colors  long      $01_03_01_03            ' Horizontal sync color mask.
vsync_colors  long      $00_02_00_02            ' Vertical sync color mask.
all_black     long      $03_03_03_03
wipe_color    long      %00_01_01_11

'       Configuration Settings

dira0         long      0-0
vcfg0         long      0-0
frqa0         long      0-0

'       Run Time Variables

buf_addr      res       1
buf_end       res       1
buf_index     res       1
deq_addr      res       1
wipe_addr     res       1
pix2_addr     res       1
pix4_addr     res       1
counter       res       1
pixels        res       1
tile_ctr      res       1
line_ctr      res       1
pix2_ptr      res       1
pix4_ptr      res       1
alt_master    res       1
reg_mask      res       1
alt_mask      res       1
reg_colors    res       1
alt_colors    res       1
y_alt         res       1
y_ctr         res       1
wipe          res       1

'-------[ Overlay color unpacker ]---------------------------------------------

              org       0

ovl_cdriver   rdlong    cbuf_addr,par           'Get the start address of the FIFO color buffer.
              mov       cdeq_addr,cbuf_addr
              sub       cdeq_addr,#4
              mov       cwipe_addr,cbuf_addr
              sub       cwipe_addr,#8
              mov       cbuf_index,cbuf_addr
              mov       cbuf_index,cbuf_addr
              mov       cbuf_end,#BUF_SIZE
              shl       cbuf_end,#2
              add       cbuf_end,cbuf_addr

cmain_lp      waitpeq   cvsync_mask,cvsync_mask
              mov       cfile_addr,par          'Get the starting address of the overlay file.
              add       cfile_addr,#4           
              rdlong    cacc,cfile_addr         'Read the offset for the color swap data.
              add       cfile_addr,cacc         'Add offset to address.
              rdlong    cacc,cfile_addr
              add       cfile_addr,cacc

              mov       cfile_ptr,cfile_addr    'Move pointer to the beginning of the color swap data.
              mov       palette,#0              'Initialize palette to black.
              neg       color_ptr,#1            'Initialize color point to ficticious last color.
              mov       cbuf_index,cbuf_addr

color_lp      rdbyte    color,cfile_ptr         'Read next color swap byte.
              add       cfile_ptr,#1            'Increment pointer.
              shr       color,#1 wc             'Add 2 LSBs of color + 1 to color pointer
              addx      color_ptr,#1
              shr       color,#1 wc
        if_c  add       color_ptr,#2
              cmpsub    color_ptr,#4 wc         'Is it beyond this palette?
        if_nc jmp       #continue               '  No:  Process it.

              mov       cacc,cbuf_index         '  Yes: Calculate next buffer index.
              add       cacc,#4
              cmp       cacc,cbuf_end wz       '       Over the top?
        if_z  mov       cacc,cbuf_addr          '         Yes: Reset to beginning.
        
cbuf_wait     rdlong    caccx,cdeq_addr          '       Get the dequeue pointer.
              cmp       caccx,cacc wz           '       Equal to new buffer index?
        if_z  jmp       #cbuf_wait              '         Yes: Buffer is full. Try again.

              wrlong    palette,cbuf_index      '       Write palette to buffer.
              mov       cbuf_index,cacc         '       Update buffer index with new value.

continue      mov       cacc,color_ptr          'Compute shift amount from pointer.
              shl       cacc,#3                 ' x 8
              add       cacc,#2                 ' + 2
              shl       color,cacc              'Shift color into position.
              xor       palette,color           'XOR it into palette.
              
              test      cvsync_mask,ina wz
        if_nz jmp       #color_lp
        
              jmp       #cmain_lp               

cvsync_mask   long      0-0

cbuf_addr     res       1
cbuf_end      res       1
cbuf_index    res       1
cdeq_addr     res       1
cwipe_addr    res       1
cfile_addr    res       1
cfile_ptr     res       1
cacc          res       1
caccx         res       1
palette       res       1
color_ptr     res       1
color         res       1

'-------[ Left over color-swap buffer space, if any ]--------------------------

extra_buf     long      $55555555[BUF_SIZE - ((@extra_buf - @color_buf) >> 2) #> 0] 

'-------[ File data and hub address info ]-------------------------------------

buffer_addr   long      0-0
img_data      long      0[6500] 'file      "slide17.dat" '

''-----------[ TERMS OF USE ]--------------------------------------------------
''
'' Permission is hereby granted, free of charge, to any person obtaining a copy of
'' this software and associated documentation files (the "Software"), to deal in
'' the Software without restriction, including without limitation the rights to use,
'' copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
'' Software, and to permit persons to whom the Software is furnished to do so,
'' subject to the following conditions: 
''
'' The above copyright notice and this permission notice shall be included in all
'' copies or substantial portions of the Software. 
''
'' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
'' INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
'' PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
'' HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
'' OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
'' SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
